@@ -62,6 +62,7 @@ module Codec.Archive.Tar.Types (
 
 import Data.Int      (Int64)
 import Data.Monoid   (Monoid(..))
+import Data.Semigroup as Sem
 import qualified Data.ByteString       as BS
 import qualified Data.ByteString.Char8 as BS.Char8
 import qualified Data.ByteString.Lazy  as LBS
@@ -79,7 +80,8 @@ import System.Posix.Types
 
 #ifdef TESTS
 import Test.QuickCheck
-import Control.Applicative ((<$>), pure, (<*>))
+import Control.Applicative ((<$>), (<*>), pure)
+import Data.Word (Word16)
 #endif
 
 
@@ -535,9 +537,13 @@ mapEntriesNoFail :: (Entry -> Entry) -> Entries e -> Entries e
 mapEntriesNoFail f =
   foldEntries (\entry -> Next (f entry)) Done Fail
 
+-- | @since 0.5.1.0
+instance Sem.Semigroup (Entries e) where
+  a <> b = foldEntries Next b Fail a
+
 instance Monoid (Entries e) where
-  mempty      = Done
-  mappend a b = foldEntries Next b Fail a
+  mempty  = Done
+  mappend = (Sem.<>)
 
 instance Functor Entries where
   fmap f = foldEntries Next Done (Fail . f)
@@ -559,13 +565,13 @@ instance Arbitrary Entry where
                     <*> arbitrary <*> arbitraryEpochTime <*> arbitrary
     where
       arbitraryPermissions :: Gen Permissions
-      arbitraryPermissions = fromIntegral <$> (arbitraryOctal 7 :: Gen Int)
+      arbitraryPermissions = fromIntegral <$> (arbitrary :: Gen Word16)
 
       arbitraryEpochTime :: Gen EpochTime
-      arbitraryEpochTime = fromIntegral <$> (arbitraryOctal 11 :: Gen Int)
+      arbitraryEpochTime = arbitraryOctal 11
 
   shrink (Entry path content perms author time format) =
-      [ Entry path' content' perms author' time' format 
+      [ Entry path' content' perms author' time' format
       | (path', content', author', time') <-
          shrink (path, content, author, time) ]
    ++ [ Entry path content perms' author time format
@@ -651,7 +657,11 @@ instance Arbitrary Ownership where
   arbitrary = Ownership <$> name <*> name
                         <*> idno <*> idno
     where
-      name = listOf0ToN 32 (arbitrary `suchThat` (/= '\0'))
+      -- restrict user/group to posix ^[a-z][-a-z0-9]{0,30}$
+      name = do
+        first <- choose ('a', 'z')
+        rest <- listOf0ToN 30 (oneof [choose ('a', 'z'), choose ('0', '9'), pure '-'])
+        return $ first : rest
       idno = arbitraryOctal 7
 
   shrink (Ownership oname gname oid gid) =
