@@ -16,11 +16,13 @@ import Codec.Archive.Tar.Types
 
 import Data.Char     (ord)
 import Data.List     (foldl')
+import Data.Monoid   (mempty)
 import Numeric       (showOct)
 
-import qualified Data.ByteString.Lazy as BS
-import qualified Data.ByteString.Lazy.Char8 as BS.Char8
-import Data.ByteString.Lazy (ByteString)
+import qualified Data.ByteString             as BS
+import qualified Data.ByteString.Char8       as BS.Char8
+import qualified Data.ByteString.Lazy        as LBS
+import qualified Data.ByteString.Lazy.Char8  as LBS.Char8
 
 
 -- | Create the external representation of a tar archive by serialising a list
@@ -28,22 +30,23 @@ import Data.ByteString.Lazy (ByteString)
 --
 -- * The conversion is done lazily.
 --
-write :: [Entry] -> ByteString
-write es = BS.concat $ map putEntry es ++ [BS.replicate (512*2) 0]
+write :: [Entry] -> LBS.ByteString
+write es = LBS.concat $ map putEntry es ++ [LBS.replicate (512*2) 0]
 
-putEntry :: Entry -> ByteString
+putEntry :: Entry -> LBS.ByteString
 putEntry entry = case entryContent entry of
-  NormalFile       content size -> BS.concat [ header, content, padding size ]
-  OtherEntryType _ content size -> BS.concat [ header, content, padding size ]
+  NormalFile       content size -> LBS.concat [ header, content, padding size ]
+  OtherEntryType _ content size -> LBS.concat [ header, content, padding size ]
   _                             -> header
   where
     header       = putHeader entry
-    padding size = BS.replicate paddingSize 0
+    padding size = LBS.replicate paddingSize 0
       where paddingSize = fromIntegral (negate size `mod` 512)
 
-putHeader :: Entry -> ByteString
+putHeader :: Entry -> LBS.ByteString
 putHeader entry =
-     BS.Char8.pack $ take 148 block
+     LBS.Char8.pack
+   $ take 148 block
   ++ putOct 7 checksum
   ++ ' ' : drop 156 block
 --  ++ putOct 8 checksum
@@ -63,7 +66,7 @@ putHeaderNoChkSum Entry {
   } =
 
   concat
-    [ putString  100 $ name
+    [ putBString 100 $ name
     , putOct       8 $ permissions
     , putOct       8 $ ownerId ownership
     , putOct       8 $ groupId ownership
@@ -71,50 +74,57 @@ putHeaderNoChkSum Entry {
     , putOct      12 $ modTime
     , fill         8 $ ' ' -- dummy checksum
     , putChar8       $ typeCode
-    , putString  100 $ linkTarget
+    , putBString 100 $ linkTarget
     ] ++
   case format of
   V7Format    ->
       fill 255 '\NUL'
   UstarFormat -> concat
-    [ putString    8 $ "ustar\NUL00"
+    [ putBString   8 $ ustarMagic
     , putString   32 $ ownerName ownership
     , putString   32 $ groupName ownership
     , putOct       8 $ deviceMajor
     , putOct       8 $ deviceMinor
-    , putString  155 $ prefix
+    , putBString 155 $ prefix
     , fill        12 $ '\NUL'
     ]
   GnuFormat -> concat
-    [ putString    8 $ "ustar  \NUL"
+    [ putBString   8 $ gnuMagic
     , putString   32 $ ownerName ownership
     , putString   32 $ groupName ownership
     , putGnuDev    8 $ deviceMajor
     , putGnuDev    8 $ deviceMinor
-    , putString  155 $ prefix
+    , putBString 155 $ prefix
     , fill        12 $ '\NUL'
     ]
   where
     (typeCode, contentSize, linkTarget,
      deviceMajor, deviceMinor) = case content of
-       NormalFile      _ size            -> ('0' , size, [],   0,     0)
-       Directory                         -> ('5' , 0,    [],   0,     0)
-       SymbolicLink    (LinkTarget link) -> ('2' , 0,    link, 0,     0)
-       HardLink        (LinkTarget link) -> ('1' , 0,    link, 0,     0)
-       CharacterDevice major minor       -> ('3' , 0,    [],   major, minor)
-       BlockDevice     major minor       -> ('4' , 0,    [],   major, minor)
-       NamedPipe                         -> ('6' , 0,    [],   0,     0)
-       OtherEntryType  code _ size       -> (code, size, [],   0,     0)
+       NormalFile      _ size            -> ('0' , size, mempty, 0,     0)
+       Directory                         -> ('5' , 0,    mempty, 0,     0)
+       SymbolicLink    (LinkTarget link) -> ('2' , 0,    link,   0,     0)
+       HardLink        (LinkTarget link) -> ('1' , 0,    link,   0,     0)
+       CharacterDevice major minor       -> ('3' , 0,    mempty, major, minor)
+       BlockDevice     major minor       -> ('4' , 0,    mempty, major, minor)
+       NamedPipe                         -> ('6' , 0,    mempty, 0,     0)
+       OtherEntryType  code _ size       -> (code, size, mempty, 0,     0)
 
     putGnuDev w n = case content of
       CharacterDevice _ _ -> putOct w n
       BlockDevice     _ _ -> putOct w n
       _                   -> replicate w '\NUL'
 
+ustarMagic, gnuMagic :: BS.ByteString
+ustarMagic = BS.Char8.pack "ustar\NUL00"
+gnuMagic   = BS.Char8.pack "ustar  \NUL"
+
 -- * TAR format primitive output
 
 type FieldWidth = Int
 
+putBString :: FieldWidth -> BS.ByteString -> String
+putBString n s = BS.Char8.unpack (BS.take n s) ++ fill (n - BS.length s) '\NUL'
+
 putString :: FieldWidth -> String -> String
 putString n s = take n s ++ fill (n - length s) '\NUL'
 
